@@ -5,6 +5,7 @@ Mount utilities.
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+from distutils.spawn import find_executable
 from collections import namedtuple
 from functools import partial
 import inspect
@@ -201,10 +202,18 @@ class Mounter(object):
         options = match_config(self._config, device, 'options', None)
         kwargs = dict(options=options)
         self._log.debug(_('mounting {0} with {1}', device, kwargs))
+        self._check_device_before_mount(device)
         mount_path = yield device.mount(**kwargs)
         self._log.info(_('mounted {0} on {1}', device, mount_path))
         yield Return(True)
 
+    def _check_device_before_mount(self, device):
+        if device.id_type == 'ntfs' and not find_executable('ntfs-3g'):
+            self._log.warn(_(
+                "Mounting NTFS device with default driver.\n"
+                "Please install 'ntfs-3g' if you experience problems or the "
+                "device is readonly."))
+
     @_sets_async_error
     @_find_device
     def unmount(self, device):
@@ -252,12 +261,16 @@ class Mounter(object):
         unlocked = yield self._unlock_from_keyfile(device)
         if unlocked:
             yield Return(True)
-        password = yield self._prompt(device)
+        password = yield self._prompt(device, self.udisks.keyfile_support)
         if password is None:
             self._log.debug(_('not unlocking {0}: cancelled by user', device))
             yield Return(False)
-        self._log.debug(_('unlocking {0}', device))
-        yield device.unlock(password)
+        if isinstance(password, bytes):
+            self._log.debug(_('unlocking {0} using keyfile', device))
+            yield device.unlock_keyfile(password)
+        else:
+            self._log.debug(_('unlocking {0}', device))
+            yield device.unlock(password)
         self._update_cache(device, password)
         self._log.info(_('unlocked {0}', device))
         yield Return(True)
@@ -285,6 +298,7 @@ class Mounter(object):
             yield Return(False)
         filename = match_config(self._config, device, 'keyfile', None)
         if filename is None:
+            self._log.debug(_('No matching keyfile rule for {}.', device))
             yield Return(False)
         try:
             with open(filename, 'rb') as f:
